/*******************************************************************************
Copyright 2016 Microchip Technology Inc. (www.microchip.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

To request to license the code under the MLA license (www.microchip.com/mla_license), 
please contact mla_licensing@microchip.com
*******************************************************************************/

/** INCLUDES *******************************************************/
#include <stdint.h>
#include <stdbool.h>

#include "system.h"

#include "usb.h"
#include "usb_device_audio.h"


/** VARIABLES ******************************************************/
/* Some processors have a limited range of RAM addresses where the USB module
 * is able to access.  The following section is for those devices.  This section
 * assigns the buffers that need to be used by the USB module into those
 * specific areas.
 */
/*
#if defined(FIXED_ADDRESS_MEMORY)
    #if defined(COMPILER_MPLAB_C18)
        #pragma udata DEVICE_AUDIO_MICROPHONE_DATA_BUFFER=DEVCE_AUDIO_MICROPHONE_DATA_BUFFER_ADDRESS
            static int16_t FrameData[8];
        #pragma udata
    #elif defined(__XC8)
        static int16_t FrameData[8] @ DEVCE_AUDIO_MICROPHONE_DATA_BUFFER_ADDRESS;
    #endif
#else
    static int16_t FrameData[8];
#endif
*/

static int16_t ReceivedDataEvenBuffer[NO_OF_SAMPLES_IN_A_USB_FRAME] DEVCE_AUDIO_SPEAKER_DATA_EVEN_BUFFER_ADDRESS;// Holds the Even Audio Data received from Host
static int16_t ReceivedDataOddBuffer[NO_OF_SAMPLES_IN_A_USB_FRAME] DEVCE_AUDIO_SPEAKER_DATA_ODD_BUFFER_ADDRESS; // Holds the Odd  Audio Data received from Host


int8_t sampleCounter = 0; // Keeps track of number of samples played.
USB_HANDLE USBTxHandle;     //USB handle.  Must be initialized to 0 at startup.
USB_HANDLE USBRxEvenHandle;
USB_HANDLE USBRxOddHandle;
bool receivedDataEvenNeedsServicingNext = false;	//TRUE means even need servicing next, FALSE means odd needs servicing next
bool newEvenData = false; // TRUE means a fresh data is arrived in the Even Buffer
bool newOddData  = false;  // TRUE means a fresh data is arrived in the Even Buffer
bool evenIsBeingPlayed = false; // TRUE means the audio frame in the Even buffer is being played currently  
bool oddIsBeingPlayed = false; // TRUE means the audio frame in the Odd buffer is being played currently 
int16_t *nextSample;  // Points to the Sample to be played next. 
uint8_t audioMute = 1; // Holds the status of Mute Control  
uint16_t temp =0;


/*********************************************************************
* Function: void APP_DeviceAudioMicrophoneInitialize(void);
*
* Overview: Initializes the demo code
*
* PreCondition: None
*
* Input: None
*
* Output: None
*
********************************************************************/
void APP_DeviceAudioMicrophoneInitialize()
{
    USBTxHandle = NULL;


   //enable the Audio Streaming OUT endpoint
    USBEnableEndpoint(USB_DEVICE_MICROPHONE_DEMO_ENDPOINT,USB_OUT_ENABLED|USB_IN_ENABLED|USB_DISALLOW_SETUP);

    //Prepare the OUT endpoints to receive the first packets from the host.
    USBRxEvenHandle = USBRxOnePacket(USB_DEVICE_MICROPHONE_DEMO_ENDPOINT,(uint8_t*)&ReceivedDataEvenBuffer,BYTES_PER_USB_FRAME); //First Audio Packet sent will arrive in the even buffer.
	USBRxOddHandle = USBRxOnePacket(USB_DEVICE_MICROPHONE_DEMO_ENDPOINT,(uint8_t*)&ReceivedDataOddBuffer,BYTES_PER_USB_FRAME); //Second Audio Packet sent will arrive in the even buffer.
	receivedDataEvenNeedsServicingNext = true;	//Used to keep track of which buffer will contain the next sequential data packet.
}

/*********************************************************************
* Function: void APP_DeviceAudioMicrophoneInitialize(void);
*
* Overview: Initializes the demo code
*
* PreCondition: None
*
* Input: None
*
* Output: None
*
********************************************************************/
void APP_DeviceAudioMicrophoneSOFHandler()
{
    
}
#if defined USB_AUDIO_FEATURE_UNIT_CONTROL_REQUESTS_HANDLER
 void UsbAudioFeatureUnitControlRequestsHandler(void)
 {
     switch (SetupPkt.W_Value.byte.HB)
     {
        case MUTE_CONTROL:
            if (SetupPkt.bRequest == SET_CUR)
            {
                USBEP0Receive((uint8_t*)&audioMute,SetupPkt.wLength,NULL);  
                if (audioMute == 0x00)
                {
                    mAudioDriverOFF();
                }
                else if (audioMute == 0x01)
                {
                    mAudioDriverON();          
                } 
            }
            else if (SetupPkt.bRequest == GET_CUR)
            {
                // Get Mute Control Status
                CtrlTrfData[0] = audioMute;
    			//CtrlTrfData[1] = 0x00;
    			USBEP0SendRAMPtr((uint8_t*)CtrlTrfData, 1, USB_EP0_NO_OPTIONS);
            }
            break;     
        case VOLUME_CONTROL:
            switch(SetupPkt.bRequest)
            {
                case SET_CUR:
                    //Set Current Volume
                    break;
                case SET_MIN:
                    break;
                case SET_MAX:
                    break;
                case SET_RES:
                    break;
                case GET_CUR:
                    // up on this request user needs to send the current volume to the host. 
			        CtrlTrfData[0] = 0x43;
			        CtrlTrfData[1] = 0x00;
			        USBEP0SendRAMPtr((uint8_t*)CtrlTrfData, 2, USB_EP0_NO_OPTIONS);
                    break;
                case GET_MIN:
			        CtrlTrfData[0] = 0;
			        CtrlTrfData[1] = 0;
			        USBEP0SendRAMPtr((uint8_t*)CtrlTrfData, 2, USB_EP0_NO_OPTIONS); 
                    break;
                case GET_MAX:
			        CtrlTrfData[0] = 0xFF;
			        CtrlTrfData[1] = 0xFF;
			        USBEP0SendRAMPtr((uint8_t*)CtrlTrfData, 2, USB_EP0_NO_OPTIONS); 
                    break;
                case GET_RES:
			        CtrlTrfData[0] = 0xFF;
			        CtrlTrfData[1] = 0xFF;
			        USBEP0SendRAMPtr((uint8_t*)CtrlTrfData, 2, USB_EP0_NO_OPTIONS); 
                    break;
                default:
                    break;
                }// end of switch(SetupPkt.bRequest)	    
            break;
        case BASS_CONTROL:
             break;
        case MID_CONTROL:
             break;
        case TREBLE_CONTROL:
             break;
        case GRAPHIC_EQUALIZER_CONTROL:
             break;
        case AUTOMATIC_GAIN_CONTROL:
             break;
        case DELAY_CONTROL:
             break;
        case BASS_BOOST_CONTROL:
             break;
        case LOUDNESS_CONTROL:
             break;
        default:
             break;     
       }  //end of switch (SetupPkt.W_Value.byte.HB)  
 
 }//end of void UsbFeatureUnitHandler(void)
 #endif 
/*********************************************************************
* Function: void APP_DeviceAudioMicrophoneTasks(void);
*
* Overview: Keeps the Custom HID demo running.
*
* PreCondition: The demo should have been initialized and started via
*   the APP_DeviceAudioMicrophoneInitialize() and APP_DeviceAudioMicrophoneStart() demos
*   respectively.
*
* Input: None
*
* Output: None
*
********************************************************************/
void APP_DeviceAudioMicrophoneTasks()
{
    unsigned char frameCounter;
        //Blink the LEDs according to the USB device status

    /* If the device is not configured yet, or the device is suspended, then
     * we don't need to run the demo since we can't send any data.
     */
    if( (USBGetDeviceState() < CONFIGURED_STATE) ||
        (USBIsDeviceSuspended() == true))
    {
        return;
    }
    
    /* If the button is held down, play the audio file or continue playing the
     * audio file.  Stop playing the file when the button is released.
     */
	if (receivedDataEvenNeedsServicingNext == true) //Check which buffer (even/odd) the next set of data is going to arrive in
	{
    	if(!USBHandleBusy(USBRxEvenHandle))//Check if the endpoint has received any data from the host.
    	{
   			//If the code gets to this point, we have received a USB frame from the host 
   			//with Audio Data in the Even Buffer. 
			newEvenData = true; 
			if (oddIsBeingPlayed == false)// Start the Audio Playback for the Even buffer only if the Odd buffer is not being played.   
			{		
				evenIsBeingPlayed = true;
				nextSample = &ReceivedDataEvenBuffer[0];// load the start address of the Audio data for Playback.   
				sampleCounter =0;
				mStartAudio(); // Unmask the Timer2 or Output Compare 1(in case of PIC24F) interrupt to start-
				// the Audio Play back.    	
			}
        	//Get ready for next packet (this will overwrite the old data)
        	USBRxEvenHandle = USBRxOnePacket(USB_DEVICE_MICROPHONE_DEMO_ENDPOINT,(uint8_t*)&ReceivedDataEvenBuffer,BYTES_PER_USB_FRAME);
			receivedDataEvenNeedsServicingNext = false; 
		}
    }
	else //else Odd Endpoint needs servicing next
	{
		if(!USBHandleBusy(USBRxOddHandle))//Check if the endpoint has received any data from the host.
		{
			//If the code gets to this point, we have received a USB frame from the host 
   			//with Audio Data in the Even Buffer. 
			newOddData = true;
			if (evenIsBeingPlayed == false)// Start the Audio Playback for the Odd buffer only if the Even buffer is not being played.   
			{
				oddIsBeingPlayed = true;
				nextSample = &ReceivedDataOddBuffer[0];// load the start address of the Audio data for Playback.   
				sampleCounter =0;
				mStartAudio();// Unmask the Timer2 or Output Compare 1(in case of PIC24F) interrupt to start-
				// the Audio Play back.  
			}
			//Get ready for next packet (this will overwrite the old data)
        	USBRxOddHandle = USBRxOnePacket(USB_DEVICE_MICROPHONE_DEMO_ENDPOINT,(uint8_t*)&ReceivedDataOddBuffer,BYTES_PER_USB_FRAME);
			receivedDataEvenNeedsServicingNext = true; 	
		}
	}
}


void APP_InterrkuptTasks(void)  //called in system.c
{
    if ((PWM_INTERRUPT_FLAG) && (TimerInterruptEnable))
	{	
		PWM_INTERRUPT_FLAG = 0; 
		if (sampleCounter < NO_OF_SAMPLES_IN_A_USB_FRAME)
		{
			//If the code gets to this point, we are in the middle of an audio playback. 
	        // The Smaples Received Host is PCM (i.e signed 16 bit integer). The PWM running at 48KHz supports only 8 bits/Sample. 
	        // So here we convert each 16 bits Sample into 8 bit Sample. Load the next sample to the PWM Duty Cycle Register. 
	        temp = (*nextSample) + 32768 ;
			DUTY_CYCLE = temp/256;  
			nextSample++;
			sampleCounter++;  
		}	
		else if (evenIsBeingPlayed)
		{
			// If the code gets to this point, we just finished playing Audio Samples in the Even Buffer.
			evenIsBeingPlayed =false; 
			newEvenData = false; 
			if (newOddData)//Check if Audio Samples are waiting to be played in Odd Buffer 
			{
				// We need to start the Playing the audio samples in the Odd buffer immediately. 
				temp = ReceivedDataOddBuffer[0] + 32768; // Load the first Sample.
				DUTY_CYCLE = temp/256;
				nextSample = &ReceivedDataOddBuffer[1];// Set the adress for the second sample.
				oddIsBeingPlayed = true;
				sampleCounter = 1;
			}  
		}
		else if (oddIsBeingPlayed)
		{ 
   			// If the code gets to this point, we just finished playing Audio Samples in the Odd Buffer.
   			oddIsBeingPlayed = 	false;
			newOddData = false;
			if (newEvenData)//Check if Audio Samples are waiting to be played in Even Buffer 
			{
				// We need to start the Playing the audio samples in the Odd buffer immediately. 
				temp = ReceivedDataEvenBuffer[0] + 32768; // Load the first Sample.
				DUTY_CYCLE = temp/256;
				nextSample = &ReceivedDataEvenBuffer[1];// Set the adress for the second sample.
				evenIsBeingPlayed = true; 
				sampleCounter = 1;
			}
		}
		else 
		{
   			//There is no Audio data present in the Even or Odd Buffers. So stop audio play back.
			mStopAudio(); // mask timer interrupt
			sampleCounter = 0;	 
		}					
    }	//End of Timer Interrupt
}
    

