/*******************************************************************************
Copyright 2016 Microchip Technology Inc. (www.microchip.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

To request to license the code under the MLA license (www.microchip.com/mla_license), 
please contact mla_licensing@microchip.com
*******************************************************************************/

#ifndef SYSTEM_H
#define SYSTEM_H

#include <xc.h>
#include <stdbool.h>
#include <stdint.h>

#include "buttons.h"
#include "leds.h"

//YTS ******************************************************************* begin

    //******* AUDIO PLAYBACK SETTINGS *******************************/
    #define AUDIO_SAMPLING_FREQUENCY_48000
    #define BYTES_PER_SAMPLE 2
    #define NO_OF_CHANNEL 1
    #define BYTES_PER_USB_FRAME (NO_OF_SAMPLES_IN_A_USB_FRAME * BYTES_PER_SAMPLE * NO_OF_CHANNEL) 

    #define CLOCK_FREQ 48000000

    /** I/O pin definitions ********************************************/
    #define INPUT_PIN 1
    #define OUTPUT_PIN 0

	/*** Audio    ************************************************/
	#ifdef AUDIO_SAMPLING_FREQUENCY_48000
   		#define NO_OF_SAMPLES_IN_A_USB_FRAME 48
   		#define PWM_PERIOD	(CLOCK_FREQ/48000)-1
	#elif defined AUDIO_SAMPLING_FREQUENCY_32000
   		#define NO_OF_SAMPLES_IN_A_USB_FRAME 32
   		#define PWM_PERIOD	(CLOCK_FREQ/32000)-1
	#elif defined AUDIO_SAMPLING_FREQUENCY_44100
   		#define NO_OF_SAMPLES_IN_A_USB_FRAME 44
   		#define PWM_PERIOD	(CLOCK_FREQ/44100)-1
	#endif 	

    /*** Intialize Audio Driver on the Speech Playback Card *********************************************/
 	#define mInitAudioDriver()  {TRISAbits.TRISA4=OUTPUT_PIN; LATAbits.LATA4 = 1; } 
	#define mAudioDriverON()	LATAbits.LATA4 = 0;    
	#define mAudioDriverOFF()	LATAbits.LATA4 = 1;

	/****** PWM Intialization************************************/ 
	// configure RC2 as output for PWM  
	// PWM mode, Single output, Active High
	#define mInitPWM() 			{TRISCbits.TRISC2 = OUTPUT_PIN;  PR2 = PWM_PERIOD; CCPR1L = 0; CCP1CON = 0x0c;}
	#define DUTY_CYCLE CCPR1L  // Duty Cycle register of the PWM Peripheral. 
	
	/****** Timer2 Intialization************************************/ 	
	// Enable Interrupt priority
	// Enable all unmasked low priority interrupts
	// Enable all unmasked high priority interrupts	
	// Timer2 interrupt is low priority 							
	#define mInitTimerInterrupt() {RCONbits.IPEN = 1; IPR1bits.TMR2IP = 1; INTCONbits.GIEL = 1; INTCONbits.GIEH = 1;}
	
	// Timer 2 on, Prescaler is 1, Postscaler is 1
	// unmask timer 2 interrupt
	#define mInitTimer() {TMR2 = 0; T2CON = 0x00; PIE1bits.TMR2IE = 0; T2CONbits.TMR2ON =1;}
	
	#define PWM_INTERRUPT_FLAG PIR1bits.TMR2IF
	
	#define TimerInterruptEnable PIE1bits.TMR2IE

    /****** Unmask Timer2 Interrupt ************************************/
    #define mStartAudio()  {PIE1bits.TMR2IE = 1;} // unmask timer 2 interrupt
    
    /******* Mask Timer2 Interrupt  ***********************************/
    #define mStopAudio()   {PIE1bits.TMR2IE = 0;}// mask timer 2 interrupt

//YTS ******************************************************************* end


#include "io_mapping.h"
#include "fixed_address_memory.h"

#include "usb_config.h"

#define MAIN_RETURN void

/*** System States **************************************************/
typedef enum
{
    SYSTEM_STATE_USB_START,
    SYSTEM_STATE_USB_SUSPEND,
    SYSTEM_STATE_USB_RESUME
} SYSTEM_STATE;

/*********************************************************************
* Function: void SYSTEM_Initialize( SYSTEM_STATE state )
*
* Overview: Initializes the system.
*
* PreCondition: None
*
* Input:  SYSTEM_STATE - the state to initialize the system into
*
* Output: None
*
********************************************************************/
void SYSTEM_Initialize( SYSTEM_STATE state );

/*********************************************************************
* Function: void SYSTEM_Tasks(void)
*
* Overview: Runs system level tasks that keep the system running
*
* PreCondition: System has been initalized with SYSTEM_Initialize()
*
* Input: None
*
* Output: None
*
********************************************************************/
//void SYSTEM_Tasks(void);
#define SYSTEM_Tasks()

#endif //SYSTEM_H
